# AMPL model for shale gas scheduling, see
# Target-rate Tracking for Shale-gas Multi-well Pads by Scheduled Shut-ins
# by Brage R. Knudsen, Bjarne Foss, Curtis H. Whitson, and Andrew R. Conn
# Preprints of the 8th IFAC Symposium on Advanced Control of Chemical Processes
# The International Federation of Automatic Control
# Furama Riverfront, Singapore, July 10-13, 2012
# 
# AMPL model supplied by Shaurya Sharma, NTNU, August 2013. 
# Source: Shaurya Sharma, "Mixed-Integer Nonlinear Programming Heuristics Applied To A Shale Gas 
# Production Optimization Problem", MSc. Thesis, Norwegian University of Science and Technology, 2013.

param jj := 6;	#Number of wells
param ii := 4;	#Number of states
param kk := 42;	#Horizon


set J := 1..jj;
set I := 1..ii;
set Ip := 1..ii;
set K := 1..kk ordered;
set kINT := {k in K: ord(k) < card(K)} ordered;

param DELTAK := 3*86400;	#Timestep: 3 days
param PLINE1 := 6.9;	#low pressure line upstream 100psi
param PLINE2 := 27.56;		#high pressure line downstream
param QMAX := 1.3110;


param a := 7.679576770363700e-04; 	#Regression parameters for p^2 to m(p) mapping
param b :=  0.001953130649100; 	# Regression parameters for p^2 to m(p) mapping 
param ptFixed := 6.894757;		#Fixed constant wellhead tubing pressure
param Ct := 4.262731e-002;	 	#tubing constant
param S :=  0.348678934155205;	#gravity term 

param QLOW := 1.3;   #Lower bound on total gas rate
param QUP := 2.4;	 	 #Upper bound on total gas rate
param QGC1 := 0.1296;	#Critical gas rate on line 1
param QGC2 := 0.2583;	#Critical gas rate on line 2
param M2 := 20;
param M3 := 15;

param Gp := 1.770000e-01; 	#gas price in dollars per unit

param BETA {J};
param Atilde {J,I,Ip};
param Btilde {J,I};
param mInit {J,I};

var pt {J,K}; 	#tubing pressure

var v {J,K} >= 0;	 #Substitution variables for bilinear product
var q {J,K} >= 0; 	#Actual flow rate at each timestep  
var m {J,I,K} >= 0; 	#State variable (grid pseudopressures)   
var pwf2 {J,K} >= 0;	#bottomhole pressure squared   
var mwf {J,K} >= 0;
var y1 {J,K} binary;	 #Pressure line variable: 1 if high pressure line is used
var y2 {J,K} binary; 	 #Decide whether or not to shut in
var w1 {J,K} >= 0;		 #Substitution variables for bilinear product
var w2 {J,K} >= 0;		 #Substitution variables for bilinear product

minimize obj:
		sum {j in J, k in kINT}   -Gp*(q[j,k]-0.05*v[j,k]);
subject to
		TOTALFLOWLOWERBOUND {k in K}:
			sum {j in J} v[j,k] >= QLOW;
		TOTALFLOWUPPERBOUND {k in K}:
			sum {j in J} v[j,k] <= QUP;
		RESPROXYMODEL {j in J, i in I, k in K: ord(k) < card(K)}:
			sum {ip in Ip} Atilde[j,i,ip]*m[j,ip,k+1]  - m[j,i,k] - Btilde[j,i]*q[j,k+1] = 0;
		WELLINFLOW1 {j in J, k in K}:
			q[j,k] <= BETA[j]*(m[j,1,k] - mwf[j,k]) +M2*y2[j,k];
		WELLINFLOW2 {j in J, k in K}:
			q[j,k] >= BETA[j]*(m[j,1,k] - mwf[j,k]) - M3*y2[j,k];			
		pSquaredToMconv {j in J, k in K}:
			mwf[j,k] = a*pwf2[j,k] +b;
		TUBINGEQUATION {j in J, k in K}:
			pwf2[j,k] = exp(S)*( (pt[j,k]^2) + ((1/Ct^2)*(q[j,k]^2)) );
		QWELLUPPER1 {j in J, k in K}:
			q[j,k] <= QMAX*(1-y2[j,k]);
		PRESSURELINE {j in J, k in K}:
			pt[j,k] >= PLINE1*y1[j,k] + PLINE2*(1-y1[j,k]) ;
		EXACTREFORM1 {j in J, k in K}:
			v[j,k] <= y1[j,k]*QMAX;
		EXACTREFORM2 {j in J, k in K}:
			v[j,k] >= q[j,k] + QMAX*(y1[j,k]-1);
		EXACTREFORM3 {j in J, k in K}:
			v[j,k] <= q[j,k];
		SHUTIN1 {j in J, k in kINT : ord(k) > 1}:
			1 - y1[j,k+1] - y1[j,k-1] + y1[j,k] >= 0;
		SHUTIN2 {j in J, k in kINT : ord(k) > 1}:
			 y1[j,k+1] + y1[j,k-1] - y1[j,k] >= 0;
		SHUTIN3 {j in J, k in kINT : ord(k) > 1}:
			y2[j,k] - y1[j,k-1] + y1[j,k] >= 0;	 
		SHUTIN4 {j in J, k in kINT : ord(k) > 1}:
			y2[j,k+1] + y2[j,k-1] - y2[j,k] >= 0;
		SHUTIN5 {j in J, k in kINT : ord(k) > 1}:
			1 - y2[j,k+1] - y2[j,k-1] + y2[j,k] >= 0;			
		CRITICALRATE {j in J, k in K}:
			q[j,k] >= QGC1*w1[j,k] + QGC2*w2[j,k];
		CRITICALRATE11 {j in J, k in K}:
			w1[j,k] <= 1;
		CRITICALRATE12 {j in J, k in K}:
			w1[j,k]  <= y1[j,k]; 
		CRITICALRATE13 {j in J, k in K}:
			w1[j,k] <= (1-y2[j,k]);
		CRITICALRATE14 {j in J, k in K}:		
			w1[j,k] >= y1[j,k] - y2[j,k];
		CRITICALRATE21 {j in J, k in K}:	
			w2[j,k] <= 1;
		CRITICALRATE22 {j in J, k in K}:	
			w2[j,k] <= (1-y1[j,k]);
		CRITICALRATE23 {j in J, k in K}:	
			w2[j,k] <= (1-y2[j,k]);
		CRITICALRATE24 {j in J, k in K}:	
			w2[j,k] >= 1-y1[j,k] - y2[j,k];
